@@ -56,10 +56,20 @@ struct _PersonalGprsMonPrivate
     GtkWidget *contents;
 	
 	GtkWidget *titleLabel;
+	GtkWidget *titleImage;
+	GtkWidget *titleBox;
+	
 	GtkWidget *uploadLabel;
+	GtkWidget *uploadImage;
 	GtkWidget *uploadContent;
+	GtkWidget *expandedUploadContent;
 	GtkWidget *downloadLabel;
+	GtkWidget *downloadImage;
 	GtkWidget *downloadContent;
+	GtkWidget *expandedDownloadContent;
+	
+	GtkWidget *expandedBox;
+	GtkWidget *compactBox;
 	
 	GtkWidget *resetLabel;
 	GtkWidget *resetContent;
@@ -70,6 +80,7 @@ struct _PersonalGprsMonPrivate
 	gint refreshSeconds;
 	gboolean binaryPrefix;
 	gboolean showResetDate;
+	gboolean compactMode;
 	gdouble version;
 	
 	gboolean isPressed;
@@ -113,6 +124,12 @@ void personal_gprs_mon_read_settings ( PersonalGprsMon *self )
 			g_error_free (error);
 			error = NULL;
 		}
+		self->priv->compactMode = g_key_file_get_boolean (keyFile, "config", "compactMode", &error);
+		if (error) {
+			self->priv->compactMode = FALSE;
+			g_error_free (error);
+			error = NULL;
+		}
 		// Introduced in version 0.4-1 to allow change in refresh intervals
 		self->priv->version = g_key_file_get_double (keyFile, "config", "version", &error);
 		if (error) {
@@ -124,6 +141,7 @@ void personal_gprs_mon_read_settings ( PersonalGprsMon *self )
         self->priv->refresh = 1;
 		self->priv->binaryPrefix = FALSE;
 		self->priv->showResetDate = FALSE;
+		self->priv->compactMode = FALSE;
 		self->priv->version = 0.3;
     }
 	
@@ -176,6 +194,7 @@ void personal_gprs_mon_write_settings (PersonalGprsMon *self)
 	g_key_file_set_integer (keyFile, "config", "refresh", self->priv->refresh);
 	g_key_file_set_boolean (keyFile, "config", "binaryPrefix", self->priv->binaryPrefix);
 	g_key_file_set_boolean (keyFile, "config", "showResetDate", self->priv->showResetDate);
+	g_key_file_set_boolean (keyFile, "config", "compactMode", self->priv->compactMode);
 	g_key_file_set_double (keyFile, "config", "version", 0.4);
 	
 	filename = g_strconcat (HOME_DIR, "/.personal_gprs_mon", NULL);
@@ -216,52 +235,89 @@ void personal_gprs_mon_content_create (PersonalGprsMon *self)
 	GtkWidget *event = gtk_event_box_new ();
 	gtk_event_box_set_visible_window (GTK_EVENT_BOX (event), FALSE);
 	self->priv->contents = gtk_vbox_new (FALSE, 0);
-	GtkWidget *bigHBox = gtk_hbox_new (TRUE, 0);
+	self->priv->compactBox = gtk_hbox_new (TRUE, 0);
 	gtk_container_set_border_width (GTK_CONTAINER (self->priv->contents), 0);
 	GtkSizeGroup *group = GTK_SIZE_GROUP (gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL));
-
 	GtkIconTheme *iconTheme = gtk_icon_theme_get_default ();
-	if (gtk_icon_theme_has_icon (iconTheme, "personal-gprs-upload")) {
-		self->priv->uploadLabel = gtk_image_new_from_icon_name ("personal-gprs-upload", GTK_ICON_SIZE_SMALL_TOOLBAR);
+	
+	if (gtk_icon_theme_has_icon (iconTheme, "personal-gprs-mon")) {
+		self->priv->titleImage = gtk_image_new_from_icon_name ("personal-gprs-mon", GTK_ICON_SIZE_BUTTON);
 	} else {
-		self->priv->uploadLabel = gtk_image_new_from_file ("/usr/share/icons/hicolor/26x26/hildon/personal-gprs-upload.png");
+		self->priv->titleImage = gtk_image_new_from_file ("/usr/share/icons/hicolor/64x64/hildon/personal-gprs-mon.png");
 	}
-	gtk_image_set_pixel_size (GTK_IMAGE (self->priv->uploadLabel), 22);
-	//gtk_size_group_add_widget (group, self->priv->uploadLabel);
+    self->priv->titleLabel = gtk_label_new ("Dataplan Monitor");
+	hildon_helper_set_logical_font (self->priv->titleLabel, "EmpSystemFont");
+	self->priv->titleBox = gtk_hbox_new (FALSE, 0);
+	gtk_container_set_border_width (GTK_CONTAINER (self->priv->titleBox), 0);
+	gtk_box_pack_start (GTK_BOX (self->priv->titleBox), self->priv->titleImage, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (self->priv->titleBox), self->priv->titleLabel, TRUE, TRUE, 0);
+	gtk_misc_set_alignment (GTK_MISC (self->priv->titleImage), 0.5, 1);
+	gtk_misc_set_padding (GTK_MISC (self->priv->titleImage), 2, 0);
+	gtk_misc_set_alignment (GTK_MISC (self->priv->titleLabel), 0, 0.8);	
+	
+	if (gtk_icon_theme_has_icon (iconTheme, "personal-gprs-upload")) {
+		self->priv->uploadImage = gtk_image_new_from_icon_name ("personal-gprs-upload", GTK_ICON_SIZE_SMALL_TOOLBAR);
+	} else {
+		self->priv->uploadImage = gtk_image_new_from_file ("/usr/share/icons/hicolor/26x26/hildon/personal-gprs-upload.png");
+	}
+	gtk_image_set_pixel_size (GTK_IMAGE (self->priv->uploadImage), 22);
+	self->priv->uploadLabel = gtk_label_new ("Uploaded");
+	gtk_size_group_add_widget (group, self->priv->uploadLabel);
 	self->priv->uploadContent = gtk_label_new ("0.000 B");
 	gtk_label_set_markup (GTK_LABEL (self->priv->uploadContent), "0<small><sup>.000</sup></small> B");
+	self->priv->expandedUploadContent = gtk_label_new ("0.000 B");
 	GtkWidget *uploadBox = gtk_hbox_new (FALSE, 0);
-	gtk_box_pack_start (GTK_BOX (uploadBox), self->priv->uploadLabel, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (uploadBox), self->priv->uploadImage, FALSE, FALSE, 0);
 	gtk_box_pack_start (GTK_BOX (uploadBox), self->priv->uploadContent, TRUE, TRUE, 0);
-	//hildon_helper_set_logical_font (self->priv->uploadLabel, "SmallSystemFont");
-	//hildon_helper_set_logical_font (self->priv->uploadContent, "SmallSystemFont");
+	hildon_helper_set_logical_font (self->priv->uploadLabel, "SmallSystemFont");
+	hildon_helper_set_logical_font (self->priv->expandedUploadContent, "SmallSystemFont");
 	hildon_helper_set_logical_color (self->priv->uploadContent, GTK_RC_FG, GTK_STATE_NORMAL, "ActiveTextColor");
+	hildon_helper_set_logical_color (self->priv->expandedUploadContent, GTK_RC_FG, GTK_STATE_NORMAL, "ActiveTextColor");
 	gtk_misc_set_alignment (GTK_MISC (self->priv->uploadLabel), 0, 0.5);
-	//gtk_misc_set_padding (GTK_MISC (self->priv->uploadLabel), HILDON_MARGIN_DEFAULT, HILDON_MARGIN_HALF);
+	gtk_misc_set_padding (GTK_MISC (self->priv->uploadLabel), HILDON_MARGIN_DEFAULT, HILDON_MARGIN_HALF);
 	gtk_misc_set_alignment (GTK_MISC (self->priv->uploadContent), 1, 1);
+	gtk_misc_set_alignment (GTK_MISC (self->priv->expandedUploadContent), 1, 1);
 	gtk_misc_set_padding (GTK_MISC (self->priv->uploadContent), HILDON_MARGIN_DEFAULT, HILDON_MARGIN_HALF);
-	gtk_box_pack_start (GTK_BOX (bigHBox), uploadBox, TRUE, TRUE, 0);
+	gtk_misc_set_padding (GTK_MISC (self->priv->expandedUploadContent), HILDON_MARGIN_DEFAULT, HILDON_MARGIN_HALF);
+	gtk_box_pack_start (GTK_BOX (self->priv->compactBox), uploadBox, TRUE, TRUE, 0);
 	
 	if (gtk_icon_theme_has_icon (iconTheme, "personal-gprs-download")) {
-		self->priv->downloadLabel = gtk_image_new_from_icon_name ("personal-gprs-download", GTK_ICON_SIZE_SMALL_TOOLBAR);
+		self->priv->downloadImage = gtk_image_new_from_icon_name ("personal-gprs-download", GTK_ICON_SIZE_SMALL_TOOLBAR);
 	} else {
-		self->priv->downloadLabel = gtk_image_new_from_file ("/usr/share/icons/hicolor/26x26/hildon/personal-gprs-download.png");
+		self->priv->downloadImage = gtk_image_new_from_file ("/usr/share/icons/hicolor/26x26/hildon/personal-gprs-download.png");
 	}
-	gtk_image_set_pixel_size (GTK_IMAGE (self->priv->downloadLabel), 22);
-	//gtk_size_group_add_widget (group, self->priv->downloadLabel);
+	gtk_image_set_pixel_size (GTK_IMAGE (self->priv->downloadImage), 22);
+	self->priv->downloadLabel = gtk_label_new ("Downloaded");
+	gtk_size_group_add_widget (group, self->priv->downloadLabel);
 	self->priv->downloadContent = gtk_label_new ("0.000 B");
 	gtk_label_set_markup (GTK_LABEL (self->priv->downloadContent), "0<small><sup>.000</sup></small> B");
+	self->priv->expandedDownloadContent = gtk_label_new ("0.000 B");
 	GtkWidget *downloadBox = gtk_hbox_new (FALSE, 0);
-	gtk_box_pack_start (GTK_BOX (downloadBox), self->priv->downloadLabel, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (downloadBox), self->priv->downloadImage, FALSE, FALSE, 0);
 	gtk_box_pack_start (GTK_BOX (downloadBox), self->priv->downloadContent, TRUE, TRUE, 0);	
-	//hildon_helper_set_logical_font (self->priv->downloadLabel, "SmallSystemFont");
-	//hildon_helper_set_logical_font (self->priv->downloadContent, "SmallSystemFont");
+	hildon_helper_set_logical_font (self->priv->downloadLabel, "SmallSystemFont");
+	hildon_helper_set_logical_font (self->priv->expandedDownloadContent, "SmallSystemFont");
 	hildon_helper_set_logical_color (self->priv->downloadContent, GTK_RC_FG, GTK_STATE_NORMAL, "ActiveTextColor");
+	hildon_helper_set_logical_color (self->priv->expandedDownloadContent, GTK_RC_FG, GTK_STATE_NORMAL, "ActiveTextColor");
 	gtk_misc_set_alignment (GTK_MISC (self->priv->downloadLabel), 0, 0.5);
-	//gtk_misc_set_padding (GTK_MISC (self->priv->downloadLabel), HILDON_MARGIN_DEFAULT, HILDON_MARGIN_HALF);
+	gtk_misc_set_padding (GTK_MISC (self->priv->downloadLabel), HILDON_MARGIN_DEFAULT, HILDON_MARGIN_HALF);
 	gtk_misc_set_alignment (GTK_MISC (self->priv->downloadContent), 1, 1);
+	gtk_misc_set_alignment (GTK_MISC (self->priv->expandedDownloadContent), 1, 1);
 	gtk_misc_set_padding (GTK_MISC (self->priv->downloadContent), HILDON_MARGIN_DEFAULT, HILDON_MARGIN_HALF);
-	gtk_box_pack_start (GTK_BOX (bigHBox), downloadBox, TRUE, TRUE, 0);
+	gtk_misc_set_padding (GTK_MISC (self->priv->expandedDownloadContent), HILDON_MARGIN_DEFAULT, HILDON_MARGIN_HALF);
+	gtk_box_pack_start (GTK_BOX (self->priv->compactBox), downloadBox, TRUE, TRUE, 0);
+	
+	GtkWidget *expandedUploadBox = gtk_hbox_new (FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (expandedUploadBox), self->priv->uploadLabel, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (expandedUploadBox), self->priv->expandedUploadContent, TRUE, TRUE, 0);
+	
+	GtkWidget *expandedDownloadBox = gtk_hbox_new (FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (expandedDownloadBox), self->priv->downloadLabel, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (expandedDownloadBox), self->priv->expandedDownloadContent, TRUE, TRUE, 0);
+	
+	self->priv->expandedBox = gtk_vbox_new (FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (self->priv->expandedBox), expandedUploadBox, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (self->priv->expandedBox), expandedDownloadBox, FALSE, FALSE, 0);
 	
 	self->priv->resetLabel = gtk_label_new ("Last Reset");
 	gtk_size_group_add_widget (group, self->priv->resetLabel);
@@ -277,9 +333,9 @@ void personal_gprs_mon_content_create (PersonalGprsMon *self)
 	gtk_misc_set_alignment (GTK_MISC (self->priv->resetContent), 1, 0.5);
 	gtk_misc_set_padding (GTK_MISC (self->priv->resetContent), HILDON_MARGIN_DEFAULT, HILDON_MARGIN_HALF);
 	
-	//gtk_box_pack_start (GTK_BOX (self->priv->contents), titleBox, FALSE, FALSE, 0);
-	gtk_box_pack_start (GTK_BOX (self->priv->contents), bigHBox, FALSE, FALSE, 0);
-	//gtk_box_pack_start (GTK_BOX (self->priv->contents), downloadBox, FALSE, FALSE, 0);	
+	gtk_box_pack_start (GTK_BOX (self->priv->contents), self->priv->titleBox, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (self->priv->contents), self->priv->compactBox, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (self->priv->contents), self->priv->expandedBox, FALSE, FALSE, 0);	
 	gtk_box_pack_start (GTK_BOX (self->priv->contents), self->priv->resetBox, FALSE, FALSE, 0);
 	gtk_container_add (GTK_CONTAINER (event), self->priv->contents);
     gtk_box_pack_start (GTK_BOX (self->priv->homeWidget), event, FALSE, FALSE, 0);
@@ -292,6 +348,12 @@ void personal_gprs_mon_content_create (PersonalGprsMon *self)
 	if (!self->priv->showResetDate) {
 		gtk_widget_hide_all (self->priv->resetBox);
 	}
+	if (self->priv->compactMode) {
+		gtk_widget_hide (self->priv->titleBox);
+		gtk_widget_hide (self->priv->expandedBox);
+	} else {
+		gtk_widget_hide (self->priv->compactBox);
+	}
 }
 
 gboolean personal_gprs_mon_update_content (PersonalGprsMon *self)
@@ -419,17 +481,9 @@ gboolean personal_gprs_mon_update_content (PersonalGprsMon *self)
 	gtk_label_set_text (GTK_LABEL (self->priv->downloadContent), g_strdup_printf ("%d.%03d %s", downloadInt, downloadRemainder, downloadUnits));
 	gtk_label_set_markup (GTK_LABEL (self->priv->downloadContent), g_strdup_printf ("%d<small><sup>.%03d</sup></small> %s", downloadInt, downloadRemainder, downloadUnits));
 
-	
-	if (self->priv->showResetDate) {
-		gtk_widget_show_all (self->priv->resetBox);
-		gtk_widget_set_size_request (GTK_WIDGET (self), 315, 75);
-		gtk_window_resize (GTK_WINDOW (self), 315, 75);
-	} else {
-		gtk_widget_hide (self->priv->resetBox);
-		gtk_widget_set_size_request (GTK_WIDGET (self), 315, 40);
-		gtk_window_resize (GTK_WINDOW (self), 315, 40);
-	}
-		
+	gtk_label_set_text (GTK_LABEL (self->priv->expandedUploadContent), g_strdup_printf ("%d.%03d %s", uploadInt, uploadRemainder, uploadUnits));
+	gtk_label_set_text (GTK_LABEL (self->priv->expandedDownloadContent), g_strdup_printf ("%d.%03d %s", downloadInt, downloadRemainder, downloadUnits));
+
 	return TRUE;
 }
 
@@ -484,12 +538,18 @@ void personal_gprs_mon_settings (HDHomePluginItem *hitem, PersonalGprsMon *self)
 	gtk_container_add (GTK_CONTAINER (content_area), showResetDate);
 	hildon_check_button_set_active (HILDON_CHECK_BUTTON (showResetDate), self->priv->showResetDate);
 	
+	GtkWidget *compactMode = hildon_check_button_new (HILDON_SIZE_AUTO_WIDTH | HILDON_SIZE_FINGER_HEIGHT);
+	gtk_button_set_label (GTK_BUTTON (compactMode), "Use compact layout");
+	gtk_container_add (GTK_CONTAINER (content_area), compactMode);
+	hildon_check_button_set_active (HILDON_CHECK_BUTTON (compactMode), self->priv->compactMode);
+	
 	gtk_widget_show_all (dialog);
 	
 	if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT) {
 		self->priv->refresh = hildon_touch_selector_get_active (HILDON_TOUCH_SELECTOR (selector), 0);
 		self->priv->binaryPrefix = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (binaryPrefix));
 		self->priv->showResetDate = hildon_check_button_get_active (HILDON_CHECK_BUTTON (showResetDate));
+		self->priv->compactMode = hildon_check_button_get_active (HILDON_CHECK_BUTTON (compactMode));
 		
 		switch (self->priv->refresh) {
 		case 0:
@@ -522,6 +582,34 @@ void personal_gprs_mon_settings (HDHomePluginItem *hitem, PersonalGprsMon *self)
 			self->priv->eventID = g_timeout_add (self->priv->refreshSeconds*1000, (GSourceFunc)personal_gprs_mon_update_content, self);
 		}
 		
+		if (self->priv->compactMode) {
+			gtk_widget_hide (self->priv->titleBox);
+			gtk_widget_hide (self->priv->expandedBox);
+			gtk_widget_show (self->priv->compactBox);
+			if (self->priv->showResetDate) {
+				gtk_widget_show (self->priv->resetBox);
+				gtk_widget_set_size_request (GTK_WIDGET (self), 315, 75);
+				gtk_window_resize (GTK_WINDOW (self), 315, 75);
+			} else {
+				gtk_widget_hide (self->priv->resetBox);
+				gtk_widget_set_size_request (GTK_WIDGET (self), 315, 40);
+				gtk_window_resize (GTK_WINDOW (self), 315, 40);
+			}
+		} else {
+			gtk_widget_show (self->priv->titleBox);
+			gtk_widget_show (self->priv->expandedBox);
+			gtk_widget_hide (self->priv->compactBox);
+			if (self->priv->showResetDate) {
+				gtk_widget_show (self->priv->resetBox);
+				gtk_widget_set_size_request (GTK_WIDGET (self), 315, 155);
+				gtk_window_resize (GTK_WINDOW (self), 315, 155);
+			} else {
+				gtk_widget_hide (self->priv->resetBox);
+				gtk_widget_set_size_request (GTK_WIDGET (self), 315, 120);
+				gtk_window_resize (GTK_WINDOW (self), 315, 120);
+			}
+		}
+		
 		personal_gprs_mon_write_settings (self);
 		personal_gprs_mon_update_content (self);
 	}
@@ -545,6 +633,24 @@ static void personal_gprs_mon_check_desktop (GObject *gobject, GParamSpec *pspec
 		g_source_remove (self->priv->eventID);
 		self->priv->eventID = 0;
 	}
+	
+	if (self->priv->compactMode) {
+		if (self->priv->showResetDate) {
+			gtk_widget_set_size_request (GTK_WIDGET (self), 315, 75);
+			gtk_window_resize (GTK_WINDOW (self), 315, 75);
+		} else {
+			gtk_widget_set_size_request (GTK_WIDGET (self), 315, 40);
+			gtk_window_resize (GTK_WINDOW (self), 315, 40);
+		}
+	} else {
+		if (self->priv->showResetDate) {
+			gtk_widget_set_size_request (GTK_WIDGET (self), 315, 155);
+			gtk_window_resize (GTK_WINDOW (self), 315, 155);
+		} else {
+			gtk_widget_set_size_request (GTK_WIDGET (self), 315, 120);
+			gtk_window_resize (GTK_WINDOW (self), 315, 120);
+		}
+	}
 }
 
 static void personal_gprs_mon_dispose (GObject *object)
@@ -604,6 +710,11 @@ static gboolean personal_gprs_mon_expose_event (GtkWidget *widget, GdkEventExpos
 	height = widget->allocation.height;
 	x = widget->allocation.x;
 	y = widget->allocation.y;
+	
+	if (!self->priv->compactMode) {
+		height -= 16;
+		y += 16;
+	}
 
 	cairo_move_to(cr, x + radius, y);
 	cairo_line_to(cr, x + width - radius, y);
@@ -627,6 +738,12 @@ static gboolean personal_gprs_mon_expose_event (GtkWidget *widget, GdkEventExpos
 	cairo_set_line_width (cr, 1);
 	cairo_stroke (cr);
 	
+	if (!self->priv->compactMode) {
+		cairo_move_to (cr, x, 56);
+		cairo_line_to (cr, x + width, 56);
+		cairo_stroke (cr);
+	}
+	
 	cairo_destroy(cr);
 
 	return GTK_WIDGET_CLASS (personal_gprs_mon_parent_class)->expose_event (widget, event);
@@ -671,16 +788,15 @@ static void personal_gprs_mon_init (PersonalGprsMon *self)
 	hints.min_width = 315;
 	hints.min_height = 40;
 	hints.max_width = 315;
-	hints.max_height = 75;
+	hints.max_height = 155;
 	gtk_window_set_geometry_hints (GTK_WINDOW (self), self->priv->homeWidget, &hints, GDK_HINT_MIN_SIZE | GDK_HINT_MAX_SIZE);
 	
 	g_type_init ();
 	self->priv->client = gconf_client_get_default ();
 	
 	personal_gprs_mon_read_settings (self);
-	
 	personal_gprs_mon_content_create (self);
-	
+
 	g_signal_connect (self, "notify::is-on-current-desktop", G_CALLBACK (personal_gprs_mon_check_desktop), self);
 }
 
